#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

OWNER_NAME = "owner_name"
PROJECT_NAME = "project_name"
PROJECT_UUID = "project_uuid"
PROJECT_UNIQUE_NAME = "project_unique_name"
CONTEXT_PATH = "context_path"
ARTIFACTS_PATH = "artifacts_path"
ITERATION = "iteration"
NAMESPACE = "namespace"
BASE = "base"
STORE_PATH = "store_path"

RUN_ARTIFACTS_PATH = "run_artifacts_path"
RUN_OUTPUTS_PATH = "run_outputs_path"
RUN_INFO = "run_info"
STATUS = "status"
CONDITION = "condition"
NAME = "name"
UUID = "uuid"
CREATED_AT = "created_at"
COMPILED_AT = "compiled_at"
SCHEDULE_AT = "schedule_at"
STARTED_AT = "started_at"
FINISHED_AT = "finished_at"
DURATION = "duration"
CLONING_KIND = "cloning_kind"
ORIGINAL_UUID = "original_uuid"
IS_INDEPENDENT = "is_independent"
