#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.constants.globals import UNKNOWN


class EventTypes:
    ADDED = "ADDED"
    MODIFIED = "MODIFIED"
    DELETED = "DELETED"
    ERROR = "ERROR"


class PodConditions:
    READY = "Ready"
    INITIALIZED = "Initialized"
    SCHEDULED = "PodScheduled"
    UNSCHEDULABLE = "Unschedulable"

    VALUES = [READY, INITIALIZED, SCHEDULED]


class PodLifeCycle:
    CONTAINER_CREATING = "ContainerCreating"
    PENDING = "Pending"
    RUNNING = "Running"
    SUCCEEDED = "Succeeded"
    FAILED = "Failed"
    UNKNOWN = UNKNOWN

    DONE_STATUS = [FAILED, SUCCEEDED]
