#!/usr/bin/python
#
# Copyright 2018-2022 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import fields

from polyaxon.schemas.base import BaseCamelSchema, BaseConfig


class SSLSchema(BaseCamelSchema):
    enabled = fields.Bool(allow_none=True)
    secret_name = fields.Str(allow_none=True)
    path = fields.Str(allow_none=True)

    @staticmethod
    def schema_config():
        return SSLConfig


class SSLConfig(BaseConfig):
    SCHEMA = SSLSchema
    REDUCED_ATTRIBUTES = ["enabled"]

    def __init__(self, enabled=None, secret_name=None, path=None):  # noqa
        self.enabled = enabled
        self.secret_name = secret_name
        self.path = path
